#install.packages("foreign", type = "source")
library(foreign)
#install.packages("stringr", type = "source")
library(stringr)

setwd("M:SHP")


################################################        BASES              #########################################################################

SHPMP = read.spss("./Uniques_Files/SPSS/SHP_MP.sav", to.data.frame=T, use.value.labels=F, use.missings=F)
SHPMH = read.spss("./Uniques_Files/SPSS/SHP_MH.sav", to.data.frame=T, use.value.labels=F, use.missings=F) 
SHPLJ = read.spss("./Uniques_Files/SPSS/SHP_LJ.sav", to.data.frame=T, use.value.labels=F, use.missings=F) 
SHPSO = read.spss("./Uniques_Files/SPSS/SHP_SO.sav", to.data.frame=T, use.value.labels=F, use.missings=F) 
SHPCA = read.spss("./Uniques_Files/SPSS/SHP_CA.sav", to.data.frame=T, use.value.labels=F, use.missings=F) 

SHPMP = SHPMP[order(SHPMP$IDPERS),]
SHPMH = SHPMH[order(SHPMH$IDHOUS..),]
SHPLJ = SHPLJ[order(SHPLJ$IDPERS),]
SHPSO = SHPSO[order(SHPSO$IDPERS),]
SHPCA = SHPCA[order(SHPCA$IDPERS),]

years = as.character(1999:2020)          # change hier
years2 = str_sub(years,start=-2, end=-1) # two digit (from the end) years
nbwaves = length(years)
waves = 1:nbwaves
finwaves = as.character(waves[length(waves)])

save(years, years2, nbwaves, waves, finwaves, file="./SHP_Data_R0/ObjectsForLoop.Rdata") #("./ObjectsForLoop.Rdata")

for(i in 1:nbwaves)
{
	y=years
	y2=sprintf("%02s", years2)
	w=sprintf("%1d", waves)
	fw=finwaves
	eval(parse(text=sprintf("SHP%1$sP=read.spss(\"./Waves_1_%2$s/SPSS/W%3$s_%4$s/SHP%1$s_P_USER.sav\",to.data.frame=T,use.value.labels=F, use.missings=F)
				 SHP%1$sH=read.spss(\"./Waves_1_%2$s/SPSS/W%3$s_%4$s/SHP%1$s_H_USER.sav\",to.data.frame=T,use.value.labels=F, use.missings=F)
				 SHP%1$sP=SHP%1$sP[order(SHP%1$sP$IDPERS),]
				 SHP%1$sH=SHP%1$sH[order(SHP%1$sH$IDHOUS%1$s),]
				 save(SHP%1$sP,SHP%1$sH, file=\"./SHP_Data_R0/SHPPH%1$s.RData\")
				", y2[i], fw[1], w[i], y[i])))			
}



#--------------------------------------------------------------
# example syntax for file creation with SHP
#--------------------------------------------------------------------------------
#
#
#NOTES
#- Do not forget to adapt directory names before running syntaxes !!!! 
#
#
#
#---1. match individual master file (MP) on W22 individual file
#// keep only individuals living in households contacted in 2020

SHPMP20=merge(SHPMP, SHP20P, by="IDPERS", all.x = FALSE, all.y = TRUE)
SHPMP20=SHPMP20[order(SHPMP20$IDPERS),]

# for each of the years
for(i in 1:nbwaves)
{
	y=years
	y2=sprintf("%02s", years2)
	w=sprintf("%1d", waves)
	eval(parse(text=sprintf("SHPMP%1$s=merge(SHPMP, SHP%1$sP, by=\"IDPERS\", all.x = FALSE, all.y = TRUE)
				 SHPMP%1$s=SHPMP%1$s[order(SHPMP%1$s$IDPERS),]
				 save(SHPMP%1$s, file=\"./SHP_Data_R0/SHPMP%1$s.RData\")			 
				", y2[i])))		
}


#---2. match individual master file (MP) on both individual files of W1 and W22

SHPMP20=merge(SHPMP, SHP20P, by="IDPERS", all.x = FALSE, all.y = TRUE)
SHPMP20=SHPMP20[order(SHPMP20$IDPERS),]
SHPMP9920=merge(SHPMP20, SHP99P, by="IDPERS", all.x = TRUE, all.y = TRUE)

# for each of the years
for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("SHPMP%1$s=merge(SHPMP, SHP%1$sP, by=\"IDPERS\", all.x = FALSE, all.y = TRUE)
				 SHPMP%1$s=SHPMP%1$s[order(SHPMP%1$s$IDPERS),]
				 SHPMP99%1$s=merge(SHPMP%1$s, SHP99P, by=\"IDPERS\", all.x = TRUE, all.y = TRUE)
				 save(SHPMP99%1$s, file=\"./SHP_Data_R0/SHPMP99%1$s.RData\")							 
				", y2[i])))	
}

#---3. match household file W21 on individual file W21 

SHP20P=SHP20P[order(SHP20P$IDHOUS20, SHP20P$IDPERS),]
SHP20H=SHP20H[order(SHP20H$IDHOUS20),]
SHP20PH=merge(SHP20P, SHP20H, by="IDHOUS20", all.x = TRUE, all.y = FALSE) # if all=TRUE, creates duplicates with IDPERS = NA as IDINT differs between SHP20_P and SHP20_H...
SHP20PH=SHP19PH[order(SHP20PH$IDHOUS20, SHP20PH$IDPERS),]

# ! in case you wish to make (weighted) analyses on the household level with this file, 
# you should divide the household weight by the number of household members. 
# Further information is available in the user guide. #

SHP20PH$WH20CSS_cor=SHP20PH$WH20CSS/SHP20PH$NBPERS20

# for each of the years
for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("SHP%1$sP=SHP%1$sP[order(SHP%1$sP$IDHOUS%1$s,SHP%1$sP$IDPERS),]
				 SHP%1$sH=SHP%1$sH[order(SHP%1$sH$IDHOUS%1$s),]
				 SHP%1$sPH=merge(SHP%1$sP, SHP%1$sH,by=\"IDHOUS%1$s\", all.x = TRUE, all.y = FALSE) 
				 SHP%1$sPH=SHP%1$sPH[order(SHP%1$sPH$IDHOUS%1$s, SHP%1$sPH$IDPERS),]
				 save(SHP%1$sPH, file=\"./SHP_Data_R0/SHP%1$sPH.RData\")				 
				", y2[i])))		
}


#---4. match all individual unique files (WA) on individual file of W22

SHP20PMP=merge(SHP20P,SHPMP, by="IDPERS", all.x = TRUE, all.y = FALSE)
SHP20PMPLJ=merge(SHP20PMP,SHPLJ, by="IDPERS", all.x = TRUE, all.y = FALSE)
SHP20PMPLJSO=merge(SHP20PMPLJ,SHPSO, by="IDPERS", all.x = TRUE, all.y = FALSE)
SHP20PMPLJSOCA=merge(SHP20PMPLJSO,SHPCA, by="IDPERS", all.x = TRUE, all.y = FALSE)

# OR

SHP20PWA=Reduce(function(x, y) merge(x, y, all.x=TRUE), # keeps only individuals from SHP20P
                  list(SHP20P, SHPMP, SHPLJ, SHPSO, SHPCA))
dim(SHP20PWA)

# for each of the years
for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("SHP%1$sPWA=Reduce(function(x, y) merge(x, y, all.x=TRUE),list(SHP%1$sP, SHPMP, SHPLJ, SHPLJ, SHPCA))
				 save(SHP%1$sPWA, file=\"./SHP_Data_R0/SHP%1$sPWA.RData\")
				", y2[i])))		
}

#---5. create an individual longitudinal file (21 waves), with MP information

SHPMPP = SHPMP

for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("SHPMPP = merge(SHPMPP, SHP%1$sP, by=\"IDPERS\",all.x = TRUE, all.y = TRUE)
				",y2[i])))
}

save(SHPMPP, file="./SHP_Data_R0/SHPMPP.RData")

#---6. create a household longitudinal file (22 waves)

SHPMHH = SHPMH

for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("names(SHP%1$sH)[names(SHP%1$sH)==\"IDHOUS%1$s\"]=\"IDHOUS..\"
				 SHPMHH = merge(SHPMHH, SHP%1$sH, by=\"IDHOUS..\",all.x = TRUE, all.y = TRUE)
				 names(SHP%1$sH)[names(SHP%1$sH)==\"IDHOUS..\"]=\"IDHOUS%1$s\"
				", y2[i])))
}

save(SHPMHH, file="./SHP_Data_R0/SHPMHH.RData")

#---7. create a longitudinal file (22 waves) with all the individual and household files, with MP information

#determine the IDHOUS.. for merge

SHPMPP$IDHOUS..=SHPMPP$IDHOUS99.x
for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("for (j in 1:dim(SHPMPP)[1])
				{
					if (SHPMPP$IDHOUS%1$s.x[j]>0) SHPMPP$IDHOUS..[j]=SHPMPP$IDHOUS%1$s.x[j]
				}
				", y2[i])))
}

SHPMPPMHH = merge(SHPMPP, SHPMHH, by="IDHOUS..",all.x = TRUE, all.y = TRUE)
save(SHPMPPMHH, file="./SHP_Data_R0/SHPMPPMHH.RData")

#---8. create a PARTNER file for W22
#---8.1 match individual master file (MP) on W22 individual file

SHPMP20=merge(SHPMP, SHP20P, by="IDPERS", all.x = FALSE, all.y = TRUE)
SHPMP20=SHPMP20[order(SHPMP20$IDPERS),]

#---8.2 duplicate W22 individual file changing variable names to create PARTNER file 

SHPMP20PART = SHPMP20
names(SHPMP20PART) = paste(names(SHPMP20), "_p", sep="")

#---8.3 create key for matching with individual file W22

#---8.4 match PARTNER file on individual file

# version.1: keep all indiv, including with no PARTNERs
SHPMP20PARTNER = merge(SHPMP20, SHPMP20PART, by.x="IDSPOU20", by.y="IDPERS_p", all.x = TRUE, all.y = FALSE)

# version.2: keep only indiv with PARTNERs
SHPMP20PART2=SHPMP20PART[(SHPMP20PART$IDSPOU20>0),] # if missing == NA (and not num values), than: which(is.na(SHPMP20PART2$IDSPOU20_p)==F)
#SHPMP20PART2=SHPMP20PART[which(is.na(SHPMP20PART$IDSPOU20)==F),]
SHPMP20PARTNER2 = merge(SHPMP20, SHPMP20PART2, by.x="IDPERS", by.y="IDSPOU20_p", all.x = FALSE, all.y = TRUE) 

#---8.5 match household variables on PARTNER file

SHPMP20PARTNERANDH=merge(SHPMP20PARTNER, SHP20H, by.x="IDHOUS20.x_p", by.y="IDHOUS20", all.x = FALSE, all.y = FALSE)
SHPMP20PARTNERANDH2=merge(SHPMP20PARTNER2, SHP20H, by.x="IDHOUS20.x_p", by.y="IDHOUS20", all.x = FALSE, all.y = FALSE)

# for each of the years
for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("SHPMP%1$s=merge(SHPMP, SHP%1$sP, by=\"IDPERS\", all.x = FALSE, all.y = TRUE)
				 SHPMP%1$s=SHPMP%1$s[order(SHPMP%1$s$IDPERS),]
				 SHPMP%1$sPART = SHPMP%1$s
  				 names(SHPMP%1$sPART) = paste(names(SHPMP%1$s), \"_p\", sep=\"\")
				 SHPMP%1$sPARTNER = merge(SHPMP%1$s, SHPMP%1$sPART, by.x=\"IDSPOU%1$s\", by.y=\"IDPERS_p\", all.x = TRUE, all.y = FALSE)
				 SHPMP%1$sPART2=SHPMP%1$sPART[(SHPMP%1$sPART$IDSPOU%1$s>0),]
				 SHPMP%1$sPARTNER2 = merge(SHPMP%1$s, SHPMP%1$sPART2, by.x=\"IDPERS\", by.y=\"IDSPOU%1$s_p\", all.x = FALSE, all.y = TRUE)
				 SHPMP%1$sPARTNERANDH=merge(SHPMP%1$sPARTNER, SHP%1$sH, by.x=\"IDHOUS%1$s.x_p\", by.y=\"IDHOUS%1$s\", all.x = FALSE, all.y = FALSE)
				 SHPMP%1$sPARTNERANDH2=merge(SHPMP%1$sPARTNER2, SHP%1$sH, by.x=\"IDHOUS%1$s.x_p\", by.y=\"IDHOUS%1$s\", all.x = FALSE, all.y = FALSE)
				 save(SHPMP%1$sPARTNER, file=\"./SHP_Data_R0/SHPMP%1$sPARTNER.RData\")
				 save(SHPMP%1$sPARTNER2, file=\"./SHP_Data_R0/SHPMP%1$sPARTNER2.RData\")
				 save(SHPMP%1$sPARTNERANDH, file=\"./SHP_Data_R0/SHPMP%1$sPARTNERANDH.RData\")
				 save(SHPMP%1$sPARTNERANDH2, file=\"./SHP_Data_R0/SHPMP%1$sPARTNERANDH2.RData\")
				 ",y2[i])))		
}

############################################################################################################################

############## Weights

for (i in 1:nbwaves) {
    	y2 = sprintf("%02s", years2) 
 	eval(parse(text=sprintf("listvarw_p%1$s = c(\"IDPERS\", \"IDHOUS%1$s\", \"STATUS%1$s\",
						    \"WEIP%1$sTP\", \"WEIP%1$sTS\", \"WEIP%1$sLP\", \"WEIP%1$sLS\",
						    \"WP%1$sT1P\", \"WP%1$sT1S\", \"WP%1$sLP1P\", \"WP%1$sLP1S\", 
						    \"WP%1$sL1P\", \"WP%1$sL1S\", 
						    \"WI%1$sCSP\", \"WI%1$sCSS\",
						    \"WI%1$sLP1\", \"WI%1$sLS1\", \"WI%1$sLP2\", \"WI%1$sLS2\", \"WI%1$sLP3\", \"WI%1$sLS3\",
						    \"WI%1$sCSP3\", \"WI%1$sCSS3\",
						    \"WI%1$sLP99\", \"WI%1$sLS99\", \"WI%1$sLP04\", \"WI%1$sLS04\", \"WI%1$sLP13\", \"WI%1$sLS13\",
						    \"WI%1$sCSP133\", \"WI%1$sCSS133\", \"WI%1$sLP133\", \"WI%1$sLS133\",
						    \"WC%1$sCSP\", \"WC%1$sCSS\", \"WC%1$sCSP3\", \"WC%1$sCSS3\")
				 
				 varw_p%1$s = intersect(names(SHP%1$sP), listvarw_p%1$s)
                           
                           	 SHP%1$sPsubw = subset(SHP%1$sP, select=c(varw_p%1$s))
				 
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WEIP%1$sTP\"]=\"WI%1$sCSP\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WEIP%1$sTS\"]=\"WI%1$sCSS\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WEIP%1$sLP\"]=\"WI%1$sLP99\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WEIP%1$sLS\"]=\"WI%1$sLS99\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WP%1$sT1P\"]=\"WI%1$sCSP\"
        			 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WP%1$sT1S\"]=\"WI%1$sCSS\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WP%1$sLP1P\"]=\"WI%1$sLP99\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WP%1$sLS1P\"]=\"WI%1$sLS99\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WP%1$sL1P\"]=\"WI%1$sLP04\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WP%1$sL1S\"]=\"WI%1$sLS04\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WI%1$sLP1\"]=\"WI%1$sLP99\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WI%1$sLS1\"]=\"WI%1$sLS99\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WI%1$sLP2\"]=\"WI%1$sLP04\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WI%1$sLS2\"]=\"WI%1$sLS04\"
        			 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WI%1$sLP3\"]=\"WI%1$sLP13\"
				 names(SHP%1$sPsubw)[names(SHP%1$sPsubw)==\"WI%1$sLS3\"]=\"WI%1$sLS13\"

				 listvarw_h%1$s = c(\"IDHOUS%1$s\", \"WEIH%1$sTP\", \"WEIH%1$sTS\", \"WH%1$sT1P\", \"WH%1$sT1S\",
						    \"WH%1$sCSP\", \"WH%1$sCSS\", \"WH%1$sCSP3\", \"WH%1$sCSS3\")
                           
                           	 varw_h%1$s = intersect(names(SHP%1$sH), listvarw_h%1$s)
                           
                           	 SHP%1$sHsubw = subset(SHP%1$sH, select=c(varw_h%1$s))
                           	 
				 names(SHP%1$sHsubw)[names(SHP%1$sHsubw)==\"WEIH%1$sTP\"]=\"WH%1$sCSP\"
				 names(SHP%1$sHsubw)[names(SHP%1$sHsubw)==\"WEIH%1$sTS\"]=\"WH%1$sCSS\"
				 names(SHP%1$sHsubw)[names(SHP%1$sHsubw)==\"WH%1$sT1P\"]=\"WH%1$sCSP\"
				 names(SHP%1$sHsubw)[names(SHP%1$sHsubw)==\"WH%1$sT1S\"]=\"WH%1$sCSS\"
                          	 
				 save(SHP%1$sPsubw, SHP%1$sHsubw, file = \"./SHP_Data_R0/SHP%1$sPHsubw.RData\" )
                           	", y2[i] )))
}


########## P weights

SHPPWeights=SHPMP[,c("IDPERS", "OSM")]
for (i in 1:nbwaves) {
    	y2 = sprintf("%02s", years2) 
 	eval(parse(text=sprintf("SHPPWeights = merge(SHPPWeights,SHP%1$sPsubw,by=\"IDPERS\", all.x = TRUE, all.y = TRUE )
				 save(SHPPWeights, file = \"./SHP_Data_R0/SHPPWeights.RData\" )
                           	", y2[i] )))
}

names(SHPPWeights)

########## H weights

SHPHWeights=SHPMH[,c("IDHOUS..","IDORIGH")]

for(i in 1:nbwaves)
{
	y2=sprintf("%02s", years2)
	eval(parse(text=sprintf("names(SHP%1$sHsubw)[names(SHP%1$sHsubw)==\"IDHOUS%1$s\"]=\"IDHOUS..\"
				 SHPHWeights = merge(SHPHWeights, SHP%1$sHsubw, by=\"IDHOUS..\",all.x = TRUE, all.y = TRUE)
				 names(SHP%1$sHsubw)[names(SHP%1$sHsubw)==\"IDHOUS..\"]=\"IDHOUS%1$s\"
				 save(SHPHWeights, file = \"./SHP_Data_R0/SHPHWeights.RData\" )
				", y2[i])))
}

names(SHPHWeights)

